Method: HTTPClient#force_basic_auth=
- Defined in:
- lib/httpclient.rb
#force_basic_auth=(force_basic_auth) ⇒ Object
Turn on/off the BasicAuth force flag. Generally HTTP client must send Authorization header after it gets 401 error from server from security reason. But in some situation (e.g. API client) you might want to send Authorization from the beginning.
573 574 575 |
# File 'lib/httpclient.rb', line 573 def force_basic_auth=(force_basic_auth) @www_auth.basic_auth.force_auth = @proxy_auth.basic_auth.force_auth = force_basic_auth end |